package com.engine.kqsolution.web;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.engine.common.util.ParamUtil;
import com.engine.common.util.ServiceUtil;
import com.engine.kqsolution.service.ResourceSnipService;
import com.engine.kqsolution.service.impl.ResourceSnipServiceImpl;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import weaver.hrm.HrmUserVarify;
import weaver.hrm.User;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.StreamingOutput;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author liang.cheng
 * @Date 2024/8/29 9:22 AM
 * @Description: TODO
 * @Version 1.0
 */
public class ResourceSnipAction {

    private ResourceSnipService getService(User user) {
        return  ServiceUtil.getService(ResourceSnipServiceImpl.class, user);
    }

    @GET
    @Path("/snipshot")
    @Produces(MediaType.TEXT_PLAIN)
    public String resourceSnip(@Context HttpServletRequest request, @Context HttpServletResponse response,
                                 @QueryParam("monthDate") String monthDate,@QueryParam("companyStartDate") String companyStartDate){
        Map<String, Object> data = new HashMap<>(8);
        try {
            User user = HrmUserVarify.getUser(request, response);
            data.put("datas",getService(user).resourceSnip(monthDate,companyStartDate));
            data.put("api_status", true);
        } catch (Exception e) {
            data.put("api_status", false);
            data.put("msg", "catch exception : " + e.getMessage());
        }
        return JSONObject.toJSONString(data, SerializerFeature.DisableCircularReferenceDetect);
    }

    @GET
    @Path("/snipshot/export")
    @Produces(MediaType.APPLICATION_OCTET_STREAM)
    public Response snipshotExport(@Context HttpServletRequest request, @Context HttpServletResponse response,
                                   @QueryParam("monthDate") String monthDate,@QueryParam("companyStartDate") String companyStartDate) {
        User user = HrmUserVarify.getUser(request, response);
        XSSFWorkbook workbook = getService(user).snipshotExport(request, response, monthDate,companyStartDate);
        String fileName = "花名册快照" + monthDate;
        try {
            fileName = URLEncoder.encode(fileName + ".xlsx", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        StreamingOutput output = outputStream -> {
            workbook.write(outputStream);
            outputStream.flush();
        };
        response.setContentType("application/octet-stream");
        return Response.ok(output).header("Content-disposition", "attachment;filename=" + fileName).header("Cache-Control", "no-cache").build();
    }

    @GET
    @Path("/hrResource")
    @Produces(MediaType.TEXT_PLAIN)
    public String hrResource(@Context HttpServletRequest request, @Context HttpServletResponse response,
                               @QueryParam("companyStartDate") String companyStartDate){
        Map<String, Object> data = new HashMap<>(8);
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> param = ParamUtil.request2Map(request);
            data.put("datas",getService(user).hrResource(param));
            data.put("api_status", true);
        } catch (Exception e) {
            data.put("api_status", false);
            data.put("msg", "catch exception : " + e.getMessage());
        }
        return JSONObject.toJSONString(data, SerializerFeature.DisableCircularReferenceDetect);
    }

    @GET
    @Path("/hrResource/export")
    @Produces(MediaType.APPLICATION_OCTET_STREAM)
    public Response hrResourceExport(@Context HttpServletRequest request, @Context HttpServletResponse response,
                                   @QueryParam("companyStartDate") String companyStartDate) {
        User user = HrmUserVarify.getUser(request, response);
        Map<String, Object> param = ParamUtil.request2Map(request);
        XSSFWorkbook workbook = getService(user).hrResourceExport(request, response, param);
        String fileName = "人事花名册";
        try {
            fileName = URLEncoder.encode(fileName + ".xlsx", "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        StreamingOutput output = outputStream -> {
            workbook.write(outputStream);
            outputStream.flush();
        };
        response.setContentType("application/octet-stream");
        return Response.ok(output).header("Content-disposition", "attachment;filename=" + fileName).header("Cache-Control", "no-cache").build();
    }
}
